using System.Collections.Generic;

public class ReplayPlayCardAction : WatchPlayCardAction
{
	public ReplayPlayCardAction(BattleManagerBase battleMgr, OperateMgr operateMgr, NetworkBattleData networkBattleData)
		: base(battleMgr, operateMgr, networkBattleData)
	{
	}

	public override void StartSelect(int actingCardIndex, bool isPlayer = true)
	{
		if (isPlayer)
		{
			base.StartSelect(actingCardIndex, isPlayer);
		}
	}

	public override void StartChoiceSelect(int actingCardIndex, bool isPlayer = true)
	{
		if (isPlayer)
		{
			base.StartChoiceSelect(actingCardIndex, isPlayer);
		}
	}

	public override void StartSelectFusion(int actingCardIndex, bool isPlayer = true)
	{
		if (isPlayer)
		{
			base.StartSelectFusion(actingCardIndex, isPlayer);
		}
	}

	public override void SelectFusionIngredientCard(int cardIndex, bool isPlayer = true)
	{
		if (isPlayer)
		{
			base.SelectFusionIngredientCard(cardIndex, isPlayer);
		}
	}

	public override void CompleteSelectFusionIngredientCard(bool isPlayer)
	{
		if (isPlayer)
		{
			base.CompleteSelectFusionIngredientCard(isPlayer);
		}
	}

	public override void CancelSelect(bool isPlayer = true)
	{
		if (isPlayer)
		{
			base.CancelSelect(isPlayer);
		}
	}

	public override void CancelChoiceSelect(bool isPlayer = true)
	{
		if (isPlayer)
		{
			base.CancelChoiceSelect(isPlayer);
		}
	}

	public override void SelectCard(int selectedCardIndex, bool isSelfCard, bool isEvolve, bool isPlayer = true, bool isBurialRite = false, bool isChoiceBrave = false, bool isComplete = true)
	{
		if (isPlayer)
		{
			base.SelectCard(selectedCardIndex, isSelfCard, isEvolve, isPlayer, isBurialRite, isChoiceBrave: false, isComplete);
		}
	}

	public override void CompleteSelectCard(int selectedCardIndex, bool isSelfCard, bool isEvolve, bool isPlayer, bool isBurialRite, bool isChoiceBrave)
	{
		if (isPlayer)
		{
			base.SelectCard(selectedCardIndex, isSelfCard, isEvolve, isPlayer, isBurialRite, isChoiceBrave);
		}
	}

	public override void SelectChoiceCard(int selectedChoiceCardId, bool isEvolve = false, bool isPlayer = true, bool isComplete = false)
	{
		if (isPlayer && isComplete)
		{
			base.SelectChoiceCard(selectedChoiceCardId, isEvolve, isPlayer, isComplete);
		}
	}

	public override void WatchSelectChoiceCards(List<int> selectedChoiceCardIds, bool isEvolve = false, bool isPlayer = true, bool isComplete = false)
	{
		if (isPlayer && isComplete)
		{
			base.WatchSelectChoiceCards(selectedChoiceCardIds, isEvolve, isPlayer, isComplete);
		}
	}

	public override void CompleteChoiceCard(List<int> choiceIdList, bool isEvolveTargetSelect, bool isPlayer = true)
	{
		if (isPlayer)
		{
			CompleteChoiceDataIns = new CompleteChoiceData();
			OverrideChoicecard(isPlayer);
		}
	}

	public override void OverrideChoicecard(bool isPlayer)
	{
		if (isPlayer)
		{
			base.OverrideChoicecard(isPlayer);
		}
	}
}
